@@ -78,8 +78,8 @@ extern struct timeval	starttime;
 
 oid objid_enterprisetrap[] = { EXTENSIBLEMIB, 251 };
 oid version_id[]	   = { EXTENSIBLEMIB, AGENTID, OSTYPE };
-int enterprisetrap_len = OID_LENGTH( objid_enterprisetrap );
-int version_id_len     = OID_LENGTH( version_id );
+size_t enterprisetrap_len = OID_LENGTH( objid_enterprisetrap );
+size_t version_id_len     = OID_LENGTH( version_id );
 
 #define SNMPV2_TRAPS_PREFIX	SNMP_OID_SNMPMODULES,1,1,5
 oid  cold_start_oid[] =		{ SNMPV2_TRAPS_PREFIX, 1 };	/* SNMPv2-MIB */
@@ -93,9 +93,9 @@ oid  egp_xxx_oid[] =		{ SNMPV2_TRAPS_PREFIX, 99 };	/* ??? */
 oid  snmptrap_oid[] 	      =	{ SNMPV2_TRAP_OBJS_PREFIX, 1, 0 };
 oid  snmptrapenterprise_oid[] =	{ SNMPV2_TRAP_OBJS_PREFIX, 3, 0 };
 oid  sysuptime_oid[] 	      =	{ SNMP_OID_MIB2,1,3,0 };
-int  snmptrap_oid_len 	      =	OID_LENGTH(snmptrap_oid);
-int  snmptrapenterprise_oid_len = OID_LENGTH(snmptrapenterprise_oid);
-int  sysuptime_oid_len 	      =	OID_LENGTH(sysuptime_oid);
+size_t  snmptrap_oid_len 	      =	OID_LENGTH(snmptrap_oid);
+size_t  snmptrapenterprise_oid_len = OID_LENGTH(snmptrapenterprise_oid);
+size_t  sysuptime_oid_len 	      =	OID_LENGTH(sysuptime_oid);
 
 
 #define SNMP_AUTHENTICATED_TRAPS_ENABLED	1
@@ -677,7 +677,7 @@ static void trapOptProc(int argc, char *const *argv, int opt)
 
 void
 snmpd_parse_config_trapsess(const char *word, char *cptr) {
-    char args[MAX_ARGS][SPRINT_MAX_LEN], *argv[MAX_ARGS];
+    char *argv[MAX_ARGS];
     int argn, arg;
     struct snmp_session session, *ss;
 
@@ -685,17 +685,23 @@ snmpd_parse_config_trapsess(const char *word, char *cptr) {
     traptype = SNMP_MSG_TRAP2;
 
     /* create the argv[] like array */
-    strcpy(argv[0] = args[0], "snmpd-trapsess"); /* bogus entry for getopt() */
-    for(argn = 1; cptr && argn < MAX_ARGS;
-        cptr = copy_word(cptr, argv[argn] = args[argn++])) {
+    argv[0] = strdup("snmpd-trapsess"); /* bogus entry for getopt() */
+    for(argn = 1; cptr && argn < MAX_ARGS; argn++) {
+	argv[argn] = strdup(cptr); /* more than enough room */
+        cptr = copy_word(cptr, argv[argn]);
     }
 
     arg = snmp_parse_args(argn, argv, &session, "C:", trapOptProc);
+#ifdef XXX_BROKEN /* parse_args doesn't clone memory like it should */
+    do { free(argv[--argn]); } while (argn > 0);
+#endif
 
+    if (session.remote_port == SNMP_DEFAULT_REMPORT)
+        session.remote_port = SNMP_TRAP_PORT;
     ss = snmp_open (&session);
 
     if (!ss) {
-        config_perror("snmpd: failed to parse this line");
+        config_perror("snmpd: failed to parse this line or the remote trap receiver is down.  Pausible cause:");
         snmp_sess_perror("snmpd: snmpd_parse_config_trapsess()", &session);
         return;
     }
