@@ -4,6 +4,11 @@
  */
 
 #include <config.h>
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
 
 #include "host_res.h"
 #include "mibII/interfaces.h"
