@@ -4,8 +4,6 @@
  */
 
 #include <config.h>
-#include "mibincl.h"
-#include "util_funcs.h"
 
 #if defined(IFNET_NEEDS_KERNEL) && !defined(_KERNEL)
 #define _KERNEL 1
@@ -14,10 +12,17 @@
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
+#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
 
 #if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
 #endif
 #if HAVE_SYS_SYSCTL_H
 #ifdef _I_DEFINED_KERNEL
@@ -34,13 +39,18 @@
 #if HAVE_SYS_TCPIPSTATS_H
 #include <sys/tcpipstats.h>
 #endif
+#if HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #if HAVE_NET_IF_VAR_H
 #include <net/if_var.h>
 #endif
 #ifdef _I_DEFINED_KERNEL
 #undef _KERNEL
 #endif
+#if HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
 #if HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
 #endif
@@ -50,7 +60,9 @@
 #if HAVE_NETINET_IN_VAR_H
 #include <netinet/in_var.h>
 #endif
+#if HAVE_NETINET_IP_H
 #include <netinet/ip.h>
+#endif
 #if HAVE_NETINET_IP_VAR_H
 #include <netinet/ip_var.h>
 #endif
@@ -60,12 +72,14 @@
 #if HAVE_SYS_STREAM_H
 #include <sys/stream.h>
 #endif
+#if HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
 #if HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
 
-
+#include "tools.h"
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #else
@@ -75,6 +89,8 @@
 #include "kernel_linux.h"
 #endif
 
+#include "mibincl.h"
+#include "util_funcs.h"
 #include "system.h"
 #include "auto_nlist.h"
 
@@ -225,6 +241,11 @@ void init_ip(void)
 #define	USES_TRADITIONAL_IPSTAT
 #endif
 
+#ifdef WIN32
+#include <iphlpapi.h>
+#define IP_STAT_STRUCTURE MIB_IPSTATS
+#endif
+
 #if !defined(IP_STAT_STRUCTURE)
 #define IP_STAT_STRUCTURE	struct ipstat
 #define	USES_TRADITIONAL_IPSTAT
@@ -381,6 +402,29 @@ var_ip(struct variable *vp,
 
 #endif		/* USE_TRADITIONAL_IPSTAT */
 
+#ifdef WIN32
+       case IPFORWARDING:      return (u_char *) &ipstat.dwForwarding;
+       case IPDEFAULTTTL:      return (u_char *) &ipstat.dwDefaultTTL;
+       case IPINRECEIVES:      return (u_char *) &ipstat.dwInReceives;
+       case IPINHDRERRORS:     return (u_char *) &ipstat.dwInHdrErrors;
+       case IPINADDRERRORS:    return (u_char *) &ipstat.dwInAddrErrors;
+       case IPFORWDATAGRAMS:   return (u_char *) &ipstat.dwForwDatagrams;
+       case IPINUNKNOWNPROTOS: return (u_char *) &ipstat.dwInUnknownProtos;
+       case IPINDISCARDS:      return (u_char *) &ipstat.dwInDiscards;
+       case IPINDELIVERS:      return (u_char *) &ipstat.dwInDelivers;
+       case IPOUTREQUESTS:     return (u_char *) &ipstat.dwOutRequests;
+       case IPOUTDISCARDS:     return (u_char *) &ipstat.dwOutDiscards;
+       case IPOUTNOROUTES:     return (u_char *) &ipstat.dwOutNoRoutes;
+       case IPREASMTIMEOUT:    return (u_char *) &ipstat.dwReasmTimeout;
+       case IPREASMREQDS:      return (u_char *) &ipstat.dwReasmReqds;
+       case IPREASMOKS:        return (u_char *) &ipstat.dwReasmOks;
+       case IPREASMFAILS:      return (u_char *) &ipstat.dwReasmFails;
+       case IPFRAGOKS:         return (u_char *) &ipstat.dwFragOks;
+       case IPFRAGFAILS:       return (u_char *) &ipstat.dwFragFails;
+       case IPFRAGCREATES:     return (u_char *) &ipstat.dwFragCreates;
+       case IPROUTEDISCARDS:   return (u_char *) &ipstat.dwRoutingDiscards;
+#endif /* WIN32 */
+
 	default:
 	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ip\n", vp->magic));
     }
@@ -439,8 +483,11 @@ read_ip_stat( IP_STAT_STRUCTURE *ipstat, int magic )
     ret_value = getMibstat(MIB_IP, ipstat, sizeof(mib2_ip_t), GET_FIRST, &Get_everything, NULL);
 #endif
 
+#ifdef WIN32
+    ret_value = GetIpStatistics(ipstat);
+#endif
 
-#if !(defined(linux) || defined(solaris2))
+#if !(defined(linux) || defined(solaris2) || defined(WIN32))
     if ( magic == IPFORWARDING ) {
 
 #if defined(CAN_USE_SYSCTL) && defined(IPCTL_STATS)
