@@ -92,6 +92,20 @@ SOFTWARE.
 #if HAVE_PROCESS_H  /* Win32-getpid */
 #include <process.h>
 #endif
+#if HAVE_LIMITS_H
+#include <limits.h>
+#endif
+#if HAVE_PWD_H
+#include <pwd.h>
+#endif
+
+#ifndef PATH_MAX
+# ifdef _POSIX_PATH_MAX
+#  define PATH_MAX _POSIX_PATH_MAX
+# else
+#  define PATH_MAX 255
+# endif
+#endif
 
 #ifndef FD_SET
 typedef long    fd_mask;
@@ -183,7 +197,6 @@ extern char  *argvrestartname;
 
 #ifdef USING_SMUX_MODULE
 static int sdlist[NUM_SOCKETS], sdlen = 0;
-int smux_listen_sd;
 #endif /* USING_SMUX_MODULE */
 
 /*
@@ -234,13 +247,12 @@ static void usage(char *prog)
 	printf("-I [-]INITLIST\tList of mib modules to initialize (or not).\n");
 	printf("\t\t (run snmpd with -Dinit_mib for a list)\n");
 	printf("-l LOGFILE\tPrint warnings/messages to LOGFILE\n");
-	printf("\t\t(By default LOGFILE=%s)\n",
 #ifdef LOGFILE
-			LOGFILE
+	printf("\t\t(By default LOGFILE=%s)\n", LOGFILE);
 #else
-			"none"
+	printf("\t\t(By default LOGFILE=none)\n");
 #endif
-	      );
+
 #if HAVE_UNISTD_H
 	printf("-g \t\tChange to this gid after opening port\n");
 	printf("-u \t\tChange to this uid after opening port\n");
@@ -303,7 +315,7 @@ main(int argc, char *argv[])
 	int             arg, i;
 	int             ret;
 	int             dont_fork = 0;
-	char            logfile[SNMP_MAXBUF_SMALL];
+	char            logfile[PATH_MAX + 1] = { 0 };
 	char           *cptr, **argvptr;
 	char           *pid_file = NULL;
         char            buf[SPRINT_MAX_LEN];
@@ -315,10 +327,8 @@ main(int argc, char *argv[])
 	int             uid=0, gid=0;
         int             agent_mode=-1;
 
-	logfile[0]		= 0;
-
 #ifdef LOGFILE
-	strcpy(logfile, LOGFILE);
+	strncpy(logfile, LOGFILE, PATH_MAX);
 #endif
 
 #ifdef NO_ROOT_ACCESS
@@ -432,6 +442,7 @@ main(int argc, char *argv[])
                   if (++arg == argc)
                     usage(argv[0]);
                   pid_file = argv[arg];
+		  break;
 
                 case 'a':
 		  log_addresses++;
@@ -446,9 +457,16 @@ main(int argc, char *argv[])
                   break;
 
                 case 'l':
-                  if (++arg == argc)
+                  if (++arg == argc) {
                     usage(argv[0]);
-                  strcpy(logfile, argv[arg]);
+		  }
+		  if (strlen(argv[arg]) > PATH_MAX) {
+		    fprintf(stderr,
+			    "%s: logfile path too long (limit %d chars)\n",
+			    argv[0], PATH_MAX);
+		    exit(1);
+		  }
+                  strncpy(logfile, argv[arg], PATH_MAX);
                   break;
 
                 case 'L':
@@ -471,7 +489,24 @@ main(int argc, char *argv[])
 #if HAVE_UNISTD_H
                 case 'u':
                   if (++arg == argc) usage(argv[0]);
-                  ds_set_int(DS_APPLICATION_ID, DS_AGENT_USERID,atoi(argv[arg]));
+		  { char *ecp;
+		    int uid;
+		    uid = strtoul(argv[arg], &ecp, 10);
+		    if (*ecp) {
+#if HAVE_GETPWNAM && HAVE_PWD_H
+		      struct passwd *info;
+		      info = getpwnam(argv[arg]);
+		      if (info) uid = info->pw_uid;
+		      else {
+#endif
+			fprintf(stderr, "Bad user id: %s\n", argv[arg]);
+			exit(1);
+#if HAVE_GETPWNAM && HAVE_PWD_H
+		      }
+#endif
+		    }
+		  ds_set_int(DS_APPLICATION_ID, DS_AGENT_USERID, uid);
+		}
                   break;
                 case 'g':
                   if (++arg == argc) usage(argv[0]);
@@ -583,6 +618,11 @@ main(int argc, char *argv[])
     }
 #endif
 
+    /*  Honor selection of standard error output.  */
+    if (!stderr_log) {
+      snmp_disable_stderrlog();
+    }
+
     SOCK_STARTUP;
     init_agent("snmpd");		/* do what we need to do first. */
     init_mib_modules();
@@ -623,7 +663,11 @@ main(int argc, char *argv[])
 #ifdef HAVE_SETGID
 	if ((gid = ds_get_int(DS_APPLICATION_ID, DS_AGENT_GROUPID)) != 0) {
 		DEBUGMSGTL(("snmpd", "Changing gid to %d.\n", gid));
-		if (setgid(gid)==-1) {
+		if (setgid(gid)==-1
+#ifdef HAVE_SETGROUPS
+		 || setgroups(1, &gid)==-1
+#endif
+		) {
 			snmp_log_perror("setgid failed");
 			if (!ds_get_boolean(DS_APPLICATION_ID, DS_AGENT_NO_ROOT_ACCESS))
 			    exit(1);
@@ -642,10 +686,6 @@ main(int argc, char *argv[])
 #endif
 #endif
 
-	/* honor selection of standard error output */
-	if (!stderr_log)
-		snmp_disable_stderrlog();
-
 	/* we're up, log our version number */
 	snmp_log(LOG_INFO, "UCD-SNMP version %s\n", VersionInfo);
 
