@@ -166,7 +166,7 @@ int _asn_bitstring_check(const char * str, u_long asn_length, u_char datum)
     char ebuf[128];
 
     if (asn_length < 1){
-	sprintf(ebuf,"%s: length %d too small", str, (int)asn_length);
+	sprintf(ebuf,"%s: length %d too small", str, (int) asn_length);
 	ERROR_MSG(ebuf);
 	return 1;
     }
@@ -803,6 +803,12 @@ asn_parse_length(u_char  *data,
 		*length <<= 8;
 		*length |= *data++;
 	}
+	if ((long) *length < 0) {
+	    snprintf(ebuf, sizeof(ebuf),
+	    	"%s: negative data length %ld\n", errpre, (long) *length);
+	    ERROR_MSG(ebuf);
+	    return NULL;
+	}
 	return data;
     } else { /* short asnlength */
 	*length = (long)lengthbyte;
@@ -2532,5 +2538,4 @@ u_char	*asn_rbuild_signed_int64 (u_char *data,
 }
 #endif /* OPAQUE_SPECIAL_TYPES */
 
-u_char	*asn_rbuild_bitstring (u_char *, size_t *, u_char, u_char *, size_t);
 #endif
