@@ -101,6 +101,9 @@ typedef long	fd_mask;
 #define PARTY_MIB_BASE	 ".1.3.6.1.6.3.3.1.3.127.0.0.1.1"
 #define CONTEXT_MIB_BASE ".1.3.6.1.6.3.3.1.4.127.0.0.1.1"
 
+/* Prototype definitions */
+static int snmp_synch_input(int op, struct snmp_session *session, int reqid,
+                                struct snmp_pdu *pdu, void *magic);
 
 struct snmp_pdu *
 snmp_pdu_create(int command)
@@ -139,8 +142,7 @@ struct variable_list* snmp_add_null_var(struct snmp_pdu * pdu,
 }
 
 
-
-int
+static int
 snmp_synch_input(int op,
 		 struct snmp_session *session,
 		 int reqid,
